<?php
/**
 * @file
 * Contains support form for miniOrange 2FA Login
 *     Module.
 */

namespace Drupal\miniorange_2fa\Form;

use Drupal\Core\Url;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\miniorange_2fa\MiniorangeUser;
use Drupal\miniorange_2fa\MoAuthUtilities;
use Drupal\miniorange_2fa\AuthenticationType;
use Drupal\miniorange_2fa\AuthenticationAPIHandler;
use Drupal\miniorange_2fa\MiniorangeCustomerProfile;


/**
 *  Showing Support form info.
 */
class MoAuthConfigureBackupMethod extends FormBase
{
    public function getFormId()
    {
        return 'miniorange_configure_backup_method';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {

        $variables_and_values = array(
            'mo_auth_2fa_kba_questions'
        );
        $mo_db_values = MoAuthUtilities::miniOrange_set_get_configurations($variables_and_values, 'GET');

        if ($mo_db_values['mo_auth_2fa_kba_questions'] !== 'Allowed') {
            $form['mo_auth_kba_error_label'] = array(
                '#type' => 'label',
                '#title' => t('<h1>Access denied</h1>You are not authorized to access this page'),
            );
            $form['#attached']['library'][] = 'core/drupal.dialog.ajax';
            return $form;
        }

        $utilities = new MoAuthUtilities();
        $questionSetOne = $utilities->mo_get_kba_questions('ONE');
        $questionSetTwo = $utilities->mo_get_kba_questions('TWO');

        $form['mo_auth_kba_label'] = array(
            '#type' => 'label',
            '#title' => t('Please choose your security questions (KBA) and answer those:'),
        );

        $form['mo_auth_question1'] = array(
            '#type' => 'select',
            '#title' => t('Question 1'),
            '#options' => $questionSetOne,
            '#attributes' => array(
                'style' => 'width:90%;',
            ),
            '#required' => TRUE,
        );
        $form['mo_auth_answer1'] = array(
            '#type' => 'textfield',
            '#attributes' => array(
                'placeholder' => t('Enter your answer'),
                'style' => 'width:90%;',
            ),
            '#required' => TRUE,
        );
        $form['mo_auth_question2'] = array(
            '#type' => 'select',
            '#title' => t('Question 2'),
            '#options' => $questionSetTwo,
            '#attributes' => array(
                'style' => 'width:90%;',
            ),
            '#required' => TRUE,
        );

        $form['mo_auth_answer2'] = array(
            '#type' => 'textfield',
            '#attributes' => array(
                'placeholder' => t('Enter your answer'),
                'style' => 'width:90%;',
            ),
            '#required' => TRUE,
        );

        $form['mo_auth_question3'] = array(
            '#type' => 'textfield',
            '#title' => t('Question 3'),
            '#attributes' => array(
                'placeholder' => t('Enter your custom question here'),
                'style' => 'width:90%;',
            ),
            '#required' => TRUE,
        );

        $form['mo_auth_answer3'] = array(
            '#type' => 'textfield',
            '#attributes' => array(
                'placeholder' => t('Enter your answer'),
                'style' => 'width:90%;',
            ),
            '#required' => TRUE,
        );

        $form['actions'] = array('#type' => 'actions');
        $form['actions']['send'] = [
            '#type' => 'submit',
            '#value' => $this->t('Submit'),
            '#attributes' => [
                'class' => [
                    'use-ajax',
                    'button--primary'
                ],
            ],
            '#ajax' => [
                'callback' => [$this, 'submitModalFormAjax'],
                'event' => 'click',
            ],
        ];

        $form['#attached']['library'][] = 'core/drupal.dialog.ajax';
        return $form;
    }

    public function submitModalFormAjax(array $form, FormStateInterface $form_state)
    {
        $response = new AjaxResponse();
        // If there are any form errors, AJAX replace the form.
        if ($form_state->hasAnyErrors()) {
            $response->addCommand(new ReplaceCommand('#modal_support_form', $form));
        } else {
            $form_values = $form_state->getValues();
            $utilities = new MoAuthUtilities();

            $userID = \Drupal::currentUser()->id();
            $custom_attribute = MoAuthUtilities::get_users_custom_attribute($userID);
            $email = $custom_attribute[0]->miniorange_registered_email;

            $kba = array(
                array(
                    "question" => $form_values['mo_auth_question1'],
                    "answer" => $form_values['mo_auth_answer1']
                ),
                array(
                    "question" => $form_values['mo_auth_question2'],
                    "answer" => $form_values['mo_auth_answer2']
                ),
                array(
                    "question" => $form_values['mo_auth_question3'],
                    "answer" => $form_values['mo_auth_answer3']
                )
            );

            $customer = new MiniorangeCustomerProfile();
            $auth_api_handler = new AuthenticationAPIHandler($customer->getCustomerID(), $customer->getAPIKey());

            $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $email, NULL, NULL, AuthenticationType::$KBA['code']);
            $apiResponse = $auth_api_handler->register($miniorange_user, AuthenticationType::$KBA['code'], NULL, NULL, $kba);
            if (is_object($apiResponse) && $apiResponse->status == 'SUCCESS') {
                $moStatus = self::updateConfiguredMethods($userID);
                if ($moStatus) {
                    $utilities->mo_add_loggers_for_failures('Backup 2FA method has been configured successfully for - ' . $email, 'info');
                    \Drupal::messenger()->addStatus(t('Backup 2FA method has been configured successfully.'));
                } else {
                    $utilities->mo_add_loggers_for_failures('An error occurred while configuring Backup 2FA method for - ' . $email, 'error');
                    \Drupal::messenger()->addError(t('An error occurred while processing your request. Please try again after sometime.'));
                }
            } else {
                $utilities->mo_add_loggers_for_failures($apiResponse->message, 'error');
                \Drupal::messenger()->addError(t('An error occurred while processing your request. Please try again after sometime.'));
            }

            $url = Url::fromRoute('miniorange_2fa.user.mo_mfa_form', ['user' => $userID])->toString();
            $response->addCommand(new RedirectCommand($url));
        }
        return $response;
    }

    function updateConfiguredMethods($userID)
    {
        $utilities = new MoAuthUtilities();
        $customAttribute = $utilities->get_users_custom_attribute($userID);
        $configured_methods = $utilities->mo_auth_get_configured_methods($customAttribute);
        array_push($configured_methods, AuthenticationType::$KBA['code']);
        $configMethods = implode(', ', $configured_methods);
        if (count($customAttribute) > 0) {
            $database = \Drupal::database();
            $database->update('UserAuthenticationType')->fields(['configured_auth_methods' => $configMethods])->condition('uid', $userID, '=')->execute();
            return TRUE;
        }
        return FALSE;
    }

    public function validateForm(array &$form, FormStateInterface $form_state)
    {
    }

    public function submitForm(array &$form, FormStateInterface $form_state)
    {
    }
}